;====================================================================
;                Aquarius PT2 Mod Player Demo
;====================================================================
; Bruce Abbott                                    www.bhabbott.net.nz
;
; - For Mattel Aquarius with AY-3-8910
; - Demonstrates how to incorporate PlayPT2.asm into your code
;
; How to run this demo:
; 1. Load binary file into RAM at $3A00 (must have 32k RAM expansion!)
; 2. Set user address to $3A00
; 3. X=USR(X)
;
; NOTE: if running Virtual Aquarius emulator and loading code directly
;       into RAM:- make sure that the bus scramble code is set to zero!
;       PEEK(16384) should return 0.
;
; changes:
; 2015-10-17 V0.0 initial release
; 2015-10-18 V0.1 added timer

60Hz  = 1          ; 60Hz vertical blank frequency (NTSC)

        ORG  $3A00

PT2PLAY:
        CALL CLS
        CALL showhelp
_play_first:
        LD   C,0              ; C = module #1
_play_next:
        ld   a,0              ; player options (set bit0 for looping)
        ld   (PT2vars+SETUP),a
        CALL get_mod          ; HL = module data
        CALL show_mod_name
        PUSH BC
        CALL INIT             ; initialize player
        CALL reset_timer
        POP  BC
_play_frames:
        LD   B,5              ; play 5 frames
_play_frame:
        CALL wait_vb          ; wait for vertical blank pulse
        CALL get_mod          ; HL = song data
        PUSH BC
        CALL PLAY             ; play next line
        CALL Animation        ; eye candy
        call $1e80            ; KEYCHK, Get ASCII of last key pressed.
        POP  BC
        cp   $03
        jr   z,quit           ; if ^C pressed then quit
        cp   " "
        jr   z,_next_mod      ; if SPACE pressed then play next module
        LD   A,(PT2vars+SETUP)
        BIT  7,A              ; if end of module then play next
        jr   nz,_next_mod
        DJNZ   _play_frame    ; loop for 5 frames
   if   60Hz
        CALL wait_vb          ; 6th frame, slow down to 50Hz
   endif
        CALL inc_timer
        JR   _play_frames
_next_mod:
        INC  C                ; next module number
        LD   A,C
        CP   NUM_MODS         ; until all modules played
        JR   C,_play_next
        JR   _play_first

quit:   CALL MUTE             ; stop playing module and exit
        RET

; get module pointer
;  input:  C = module number
; output: HL = pointer to module data
get_mod:
        LD   A,C
        ADD  A
        LD   HL,modlist
        ADD  L
        LD   L,A
        JR   NC,$+3
        INC  H
        LD   A,(HL)
        INC  HL
        LD   H,(HL)
        LD   L,A
        RET

; Wait for Vertical Blank (PAL=50Hz, NTSC=60Hz)
wait_vb:
        IN   A,($FD)
        BIT  0,A              ; wait for end of previous vertical blank
        JR   Z,wait_vb
wait_vb_low:
        IN   A,($FD)
        BIT  0,A              ; wait for start of next vertical blank
        JR   NZ,wait_vb_low
        RET

; eye candy
Animation:
        LD   DE,$3000         ; start of character RAM
        LD   B,120
        LD   A," "
.ani    LD   (DE),A           ; blank top 3 lines of screen
        INC  DE
        DJNZ .ani
        LD   HL,PT2vars       ; PT2 player variables
        LD   DE,$3400         ; start of color RAM
        LD   BC,120
        LDIR                  ; copy variables to top 3 lines of color RAM

        LD   HL,$3000+(40*11)+10
        LD   IX,PT2vars+AYREG
        LD   (HL),"A"
        INC  HL
        LD   A,(IX+AmplA)
        CALL show_bar

        LD   (HL),"B"
        INC  HL
        LD   A,(IX+AmplB)
        CALL show_bar

        LD   (HL),"C"
        INC  HL
        LD   A,(IX+AmplC)
        CALL show_bar

        CALL show_timer

        RET


reset_timer:
        LD   HL,"00"
        LD   (time),HL
        LD   (time+2),HL
        RET

; increment the timer
inc_timer:
        LD   HL,time+3
        LD   A,(HL)
        INC  A         ; 1/10 sec + 1
        CP   "9"+1
        JR   NC,_dt_1s
        LD   (HL),A
        RET
_dt_1s  LD   (HL),"0"
        DEC  HL
        LD   A,(HL)
        INC  A
        CP   "9"+1     ; seconds +1
        JR   NC,_dt_10s
        LD   (HL),A
        RET
_dt_10s LD   (HL),"0"
        DEC  HL
        LD   A,(HL)
        INC  A         ; 10 secs +1
        CP   "6"
        JR   NC,_dt_1m
        LD   (HL),A
        RET
_dt_1m  LD   (HL),"0"
        DEC   HL
        INC  (HL)      ; munutes + 1
        RET

show_timer:
        LD   HL,time
        LD   DE,$3000+(40*18)+16
        LDI
        LD   A,":"
        LD   (DE),A
        INC  DE
        LDI
        LDI
        RET

show_mod_name:
        PUSH HL
        LD   DE,100
        ADD  HL,DE             ; HL = name in module
        EX   DE,HL
        LD   HL,$3000+(40*8)+1 ; DE = screen
        LD   A,C
        CALL show_number
        LD   (HL),"."
        INC  HL
        LD   B,34
.sm1    LD   A,(DE)
        INC  DE
        CP   " "
        JR   NC,.sm2
        LD   A," "
.sm2    LD   (HL),A
        INC  HL
        DJNZ .sm1
        POP  HL
        RET

; show decimal number 0-99
; leading zero supressed
; A  = number
; HL = screen
show_number:
        INC  A
        DAA
        LD   B,A
        RRA
        RRA
        RRA
        RRA
        AND  $0F
        JR   NZ,_sn0
        LD   A," "-"0"
_sn0    ADD  A,"0"
        LD   (HL),A
        INC  HL
        LD   A,B
        AND  $0F
        ADD  "0"
        LD   (HL),A
        INC  HL
        RET

; show horizontal bar 0-15 characters
show_bar:
.bar1   LD   B,15
        LD   C,255            ; block
        INC  A
.bar11  DEC  A
        JR   NZ,.bar12
        LD   C," "            ; blank
.bar12  LD   (HL),C
        INC  HL
        DJNZ .bar11
        LD   BC,80-16
        ADD  HL,BC
        RET

; clear screen
CLS:    LD   HL,$3000
        LD   DE,$3001
        LD   BC,999
        LD   (HL)," "
        LDIR
        RET

showhelp:
        LD   HL,help_msg
        LD   DE,$3000+(40*23)+1
        LD   BC,msg_len
        LDIR
        RET

help_msg:
        db   "<SPACE> = Next song    <CTRL>C = Quit"
msg_len = $-help_msg

; addresses of modules
modlist:
        dw   MODULE_1
        dw   MODULE_2
        dw   MODULE_3
        dw   MODULE_4
        dw   MODULE_5
        dw   MODULE_6
        dw   MODULE_7
        dw   MODULE_8
        dw   MODULE_9
        dw   MODULE_10
        dw   MODULE_11
        dw   MODULE_12
modlist_end:

NUM_MODS = (modlist_end - modlist)/2 ; number of mods in list


time     ds 4                ; clock (minutes, 10 secs, secs, tenths)
PT2vars  ds 540              ; PT2 player data

PT2_CODE = $                 ; player code
PT2_DATA = PT2vars           ; player data (540 bytes required)
     include "PlayPT2.asm"

; example modules attached to end of code
MODULE_1:
  incbin "songs/ZF4-6.pt2"
MODULE_2:
  incbin "songs/MORNING.pt2"
MODULE_3:
  incbin "songs/FAUT4_3.pt2"
MODULE_4:
  incbin "songs/MATRIX1_26.pt2"
MODULE_5:
  incbin "songs/amigoz1.pt2"
MODULE_6:
  incbin "songs/SCOOTER_20.pt2"
MODULE_7:
  incbin "songs/ZXCl9_14.pt2"
MODULE_8:
  incbin "songs/zodiac - dan 2.pt2"
MODULE_9:
  incbin "songs/THE_JAM.pt2"
MODULE_10:
  incbin "songs/RSPECY219.pt2"
MODULE_11:
  incbin "songs/I'mBlue.pt2"
MODULE_12:
  incbin "songs/ARISE.pt2"




