;=============================================================
;                   Aquarius PT2 Player
;=============================================================
;(c)2005 S.V.Bulba <vorobey@mail.khstu.ru> http://bulba.at.kz
;Ported to Aquarius by Bruce Abbott        www.bhabbott.net.nz

;Release number
Release EQU "1"
CurPosCounter = 0  ; 1 = record playing position in module

;Improvements
; max number of patterns: 85
;-correct tone portamento (see Epilogue.pt2 by Nik-O)
;-note range checking like in PT3
;-supports all PT2 subversions including PT2.4PF
;-PT2.4PF modules can be loaded to any address

;-No self-modifying code, can be put in ROM.
;-RAM can be located anywhere.

;Limitations
;-Modules like GUEST4.pt2 by Alex Job (with incorrect note+
; ornament values) sound different than with standard PT2-player.

;symbols set in main program:-
; PT2_CODE = code address (can be in RAM or ROM)
; PT2_DATA = data address (540 bytes of RAM required)

        ORG  PT2_CODE

;*************************************************************
;                  ST2 Player Code
;*************************************************************
;  input: HL = module address
;
;Entry points
;START   initialization
;START+3 play one frame (must be called 50 times per second)
;START+6 mute

START   JR INIT
        JP PLAY
        JP MUTE

INIT
;HL = AddressOfModule
        LD A,(HL)
        LD (RAM+Delay),A
        PUSH HL
        PUSH HL
        PUSH HL
        INC HL
        INC HL
        LD A,(HL)
        INC HL
        LD (SamPtrs),HL
        LD E,(HL)
        INC HL
        LD D,(HL)
        POP HL
        AND A
        SBC HL,DE
        LD (RAM+MODADDR),HL
        LD (RAM+MDADDR2),HL
        POP HL
        LD DE,67
        ADD HL,DE
        LD (OrnPtrs),HL
        LD E,32
        ADD HL,DE
        LD C,(HL)
        INC HL
        LD B,(HL)
        LD E,30
        ADD HL,DE
        LD (RAM+CrPsPtr),HL
        INC HL
        LD E,A
        ADD HL,DE
        LD (RAM+LPosPtr),HL
        POP HL
        ADD HL,BC
        LD (RAM+PatsPtr),HL

        LD HL,VAR0STRT           ; HL = start of dynamic variables RAM
        LD (HL),0                ; clear 1st byte
        LD E,L
        LD D,H
        INC DE                   ; DE = next byte
        LD BC,VAR0END-VAR0STRT-1
        LDIR                     ; clear rest of dynamic variables RAM

        LD (RAM+AdInPtA),HL      ;ptr to zero
        LD HL,$F01               ;H = CHP.Volume, L = CHP.SkpCnt
        LD (RAM+ChanA+CHP.SkpCnt),HL
        LD (RAM+ChanB+CHP.SkpCnt),HL
        LD (RAM+ChanC+CHP.SkpCnt),HL
        LD A,L
        LD (RAM+DelyCnt),A

        LD HL,EMPTYORN
        LD (RAM+ChanA+CHP.OrnPtr),HL
        LD (RAM+ChanB+CHP.OrnPtr),HL
        LD (RAM+ChanC+CHP.OrnPtr),HL

;note table creator (same as PT3 table #1)
;(c)Ivan Roshin
        LD HL,NTBL
        LD DE,RAM+NT_
        LD B,12
L1      PUSH BC
        LD C,(HL)
        INC HL
        PUSH HL
        LD B,(HL)

        PUSH DE
        EX DE,HL
        LD DE,23
        LD A,8

L2      SRL B
        RR C
        LD (HL),C
        INC HL
        LD (HL),B
        ADD HL,DE
        DEC A
        JR NZ,L2

        POP DE
        INC DE
        INC DE
        POP HL
        INC HL
        POP BC
        DJNZ L1

        LD A,$FD
        LD (RAM+NT_+$2E),A
        LD A,$0A
        LD (RAM+NT_+$5C),A

;vol table creator (same as PT3.5+ table)
;(c)Ivan Roshin

        LD HL,$11
        LD D,H
        LD E,H

        LD IX,RAM+VT_+16
        LD C,$F

INITV2  PUSH HL

        ADD HL,DE
        EX DE,HL
        SBC HL,HL

        LD B,$10
INITV1  LD A,L
        RLA
        LD A,H
        ADC A,0
        LD (IX),A
        INC IX
        ADD HL,DE
        DJNZ INITV1

        POP HL
        LD A,E
        CP $77
        JR NZ,M3
        INC E
M3      DEC C
        JR NZ,INITV2

        JP ROUT

PD_SAM  ADD A,A
        LD E,A
        LD D,0
SamPtrs EQU $+1
        LD HL,$2121
        ADD HL,DE
        LD E,(HL)
        INC HL
        LD D,(HL)
        LD HL,(RAM+MODADDR)
        ADD HL,DE
        LD (IX+CHP.SamPtr),L
        LD (IX+CHP.SamPtr+1),H
        JR PD_LOOP

PD_EOff LD (IX+CHP.Env_En),A
        JR PD_LOOP

PD_ENV  LD (IX+CHP.Env_En),16
        LD (RAM+AYREG+EnvTp),A
        LD A,(BC)
        INC BC
        LD L,A
        LD A,(BC)
        INC BC
        LD H,A
        LD (RAM+AYREG+Env),HL
        JR PD_LOOP

PD_ORN  ADD A,A
        LD E,A
        LD D,0
OrnPtrs EQU $+1
        LD HL,$2121
        ADD HL,DE
        LD E,(HL)
        INC HL
        LD D,(HL)
        LD HL,(RAM+MDADDR2)
        ADD HL,DE
        LD (IX+CHP.OrnPtr),L
        LD (IX+CHP.OrnPtr+1),H
        JR PD_LOOP

PD_SKIP INC A
        LD (IX+CHP.Skip),A
        JR PD_LOOP

PD_VOL  LD (IX+CHP.Volume),A
        JR PD_LOOP

PD_DEL  LD A,(BC)
        INC BC
        LD (RAM+Delay),A
        JR PD_LOOP

PD_GLIS SET 2,(IX+CHP.Flags)
        SET 1,(IX+CHP.Flags)
        LD A,(BC)
        INC BC
        LD (IX+CHP.TSlStp),A
        SCF
        JR PD_LP2

PTDECOD AND A

PD_LP2  EX AF,AF'

PD_LOOP LD A,(BC)
        INC BC
        ADD A,$20
        JR Z,PD_REL
        JR C,PD_SAM
        ADD A,96
        JR C,PD_NOTE
        INC A
        JR Z,PD_EOff
        ADD A,15
        JP Z,PD_QUIT
        JR C,PD_ENV
        ADD A,$10
        JR C,PD_ORN
        ADD A,$40
        JR C,PD_SKIP
        ADD A,$10
        JR C,PD_VOL
        INC A
        JR Z,PD_DEL
        INC A
        JR Z,PD_GLIS
        INC A
        JR Z,PD_PORT
        INC A
        JR Z,PD_STOP
        LD A,(BC)
        INC BC
        LD (IX+CHP.AddToN),A
        JR PD_LOOP

PD_PORT RES 2,(IX+CHP.Flags)
        SET 1,(IX+CHP.Flags)
        LD A,(BC)
        INC BC
        INC BC ;ignoring precalc delta to right sound
        INC BC
        SCF
        JR PD_LP2

PD_STOP RES 1,(IX+CHP.Flags)
        JR PD_LOOP

PD_REL  LD (IX+CHP.Flags),A
        JR PD_EXIT

PD_NOTE LD L,A
        SET 0,(IX+CHP.Flags)
        EX AF,AF'
        JR NC,NOGLISS
        BIT 2,(IX+CHP.Flags)
        JR NZ,NOPORT
        LD (IX+CHP.SlToNt),L

        PUSH BC
        LD DE,RAM+NT_
        SLA L
        LD H,0
        ADD HL,DE
        LD C,(HL)
        INC HL
        LD B,(HL)
        LD L,(IX+CHP.Note)
        SLA L
        LD H,0
        ADD HL,DE
        LD E,(HL)
        INC HL
        LD D,(HL)
        LD L,C
        LD H,B
        SBC HL,DE
        LD (IX+CHP.TnDelt),L
        LD (IX+CHP.TnDelt+1),H
        JP P,DELTP
        AND A
        JP M,SET_STP
        JR NEG_STP
DELTP   AND A
        JP P,SET_STP
NEG_STP NEG
SET_STP LD (IX+CHP.TSlStp),A
        POP BC
        JR PD_EX1

NOGLISS RES 1,(IX+CHP.Flags)
NOPORT  LD (IX+CHP.Note),L


PD_EX1  XOR A

PD_EXIT LD (IX+CHP.PsInSm),A
        LD (IX+CHP.PsInOr),A
        LD (IX+CHP.CrTnSl),A
        LD (IX+CHP.CrTnSl+1),A
PD_QUIT LD A,(IX+CHP.Skip)
        LD (IX+CHP.SkpCnt),A
        RET

CHREGS  XOR A                ; A = 0
        LD (RAM+AYREG+AmplC),A
        PUSH HL
        BIT 0,(IX+CHP.Flags)
        JP Z,CH_EXIT
        LD (RAM+CSP),SP          ; save stack pointer
        LD L,(IX+CHP.OrnPtr)
        LD H,(IX+CHP.OrnPtr+1)
        LD SP,HL
        POP DE
        LD H,A
        LD A,(IX+CHP.PsInOr)
        LD L,A
        ADD HL,SP
        INC A
        CP E
        JR C,CH_ORPS
        LD A,D
CH_ORPS LD (IX+CHP.PsInOr),A
        LD A,(IX+CHP.Note)
        ADD A,(HL)
        JP P,CH_NTP
        XOR A
CH_NTP  CP 96
        JR C,CH_NOK
        LD A,95
CH_NOK  ADD A,A
        EX AF,AF'
        LD L,(IX+CHP.SamPtr)
        LD H,(IX+CHP.SamPtr+1)
        LD SP,HL
        POP DE
        LD H,0
        LD A,(IX+CHP.PsInSm)
        LD B,A
        ADD A,A
        ADD A,B
        LD L,A
        ADD HL,SP
        LD SP,HL
        LD A,B
        INC A
        CP E
        JR C,CH_SMPS
        LD A,D
CH_SMPS LD (IX+CHP.PsInSm),A
        POP BC
        POP DE
        LD D,B
        EX AF,AF'

;ASM by AIG cannot compile
;       ADD A,NT_             ;???
;       LD L,A
;       ADC A,'NT_
;       SUB L
;       LD H,A

;Temporary
        LD L,A
        LD H,0
        LD SP,RAM+NT_
        ADD HL,SP

        LD SP,HL
        POP HL
        LD SP,(RAM+CSP)      ; restore stack pointer
        BIT 2,C
        JR Z,TSUB
        ADD HL,DE
        ADD HL,DE
TSUB    AND A
        SBC HL,DE
        LD E,(IX+CHP.CrTnSl)
        LD D,(IX+CHP.CrTnSl+1)
        ADD HL,DE
        EX (SP),HL

        BIT 1,(IX+CHP.Flags)
        JR Z,CH_AMP
        LD A,(IX+CHP.TSlStp)
        LD L,A
        ADD A,A
        SBC A,A
        LD H,A
        ADD HL,DE
        LD (IX+CHP.CrTnSl),L
        LD (IX+CHP.CrTnSl+1),H
        BIT 2,(IX+CHP.Flags)
        JR NZ,CH_AMP
        LD E,(IX+CHP.TnDelt)
        LD D,(IX+CHP.TnDelt+1)
        AND A
        JR Z,CH_STPP
        EX DE,HL
CH_STPP SBC HL,DE
        JP M,CH_AMP
        LD A,(IX+CHP.SlToNt)
        LD (IX+CHP.Note),A
        LD (IX+CHP.Flags),1
        XOR A
        LD (IX+CHP.CrTnSl),A
        LD (IX+CHP.CrTnSl+1),A

CH_AMP  LD A,B
        AND $F0
CH_VOL  OR (IX+CHP.Volume)
        RRCA
        RRCA
        RRCA
        RRCA
        LD L,A
        LD H,0
        LD DE,RAM+VT_
        ADD HL,DE
        LD A,(HL)
        OR (IX+CHP.Env_En)
        LD (RAM+AYREG+AmplC),A
        RRC C
        SBC A,A
        AND $40
        JR NZ,NONS
        LD A,C
        RRCA
        RRCA
        ADD A,(IX+CHP.AddToN)
        LD (RAM+AYREG+Noise),A
        XOR A
NONS    RRC C
        JR NC,CH_EXIT
        OR 8

CH_EXIT LD HL,RAM+AYREG+Mixer
        OR (HL)
        RRCA
        LD (HL),A
        POP HL
        RET

PLAY    XOR A
        LD (RAM+AYREG+Mixer),A
        DEC A
        LD (RAM+AYREG+EnvTp),A
        LD HL,RAM+DelyCnt
        DEC (HL)
        JP NZ,PL2
        LD HL,RAM+ChanA+CHP.SkpCnt
        DEC (HL)
        JR NZ,PL1B
        LD BC,(RAM+AdInPtA)
        LD A,(BC)
        AND A
        JR NZ,PL1A
        LD D,A
        LD HL,(RAM+CrPsPtr)
        INC HL
        LD A,(HL)
        ADD A,A                 ; end of module?
        JR NC,PLNLP
        CALL CHECKLP            ; yes, set SETUP bit7, if noloop then quit
        LD HL,(RAM+LPosPtr)
        LD A,(HL)
        ADD A,A
PLNLP   LD (RAM+CrPsPtr),HL
        ADD A,(HL)
        ADD A,A
        LD E,A
        RL D
        LD HL,(RAM+PatsPtr)
        ADD HL,DE
        LD DE,(RAM+MODADDR)
        LD (RAM+CSP),SP         ; save stack pointer
        LD SP,HL
        POP HL
        ADD HL,DE
        LD B,H
        LD C,L
        POP HL
        ADD HL,DE
        LD (RAM+AdInPtB),HL
        POP HL
        ADD HL,DE
        LD (RAM+AdInPtC),HL
        LD SP,(RAM+CSP)         ; restore stack pointer
PL1A    LD IX,RAM+ChanA
        CALL PTDECOD
        LD (RAM+AdInPtA),BC

PL1B    LD HL,RAM+ChanB+CHP.SkpCnt
        DEC (HL)
        JR NZ,PL1C
        LD IX,RAM+ChanB
        LD BC,(RAM+AdInPtB)
        CALL PTDECOD
        LD (RAM+AdInPtB),BC

PL1C    LD HL,RAM+ChanC+CHP.SkpCnt
        DEC (HL)
        JR NZ,PL1D
        LD IX,RAM+ChanC
        LD BC,(RAM+AdInPtC)
        CALL PTDECOD
        LD (RAM+AdInPtC),BC

PL1D    LD A,(RAM+Delay)
        LD (RAM+DelyCnt),A

PL2     LD IX,RAM+ChanA
        LD HL,(RAM+AYREG+ToneA)
        CALL CHREGS
        LD (RAM+AYREG+ToneA),HL
        LD A,(RAM+AYREG+AmplC)
        LD (RAM+AYREG+AmplA),A
        LD IX,RAM+ChanB
        LD HL,(RAM+AYREG+ToneB)
        CALL CHREGS
        LD (RAM+AYREG+ToneB),HL
        LD A,(RAM+AYREG+AmplC)
        LD (RAM+AYREG+AmplB),A
        LD IX,RAM+ChanC
        LD HL,(RAM+AYREG+ToneC)
        CALL CHREGS
        LD (RAM+AYREG+ToneC),HL
        JR ROUT

CHECKLP LD HL,RAM+SETUP
        SET 7,(HL)
        BIT 0,(HL)
        RET NZ       ; if loop on then continue playing current line
        POP HL       ; else pop return address, mute and quit
        LD HL,RAM+DelyCnt
        INC (HL)
        LD HL,RAM+ChanA+CHP.SkpCnt
        INC (HL)

MUTE    XOR A
        LD H,A
        LD L,A
        LD (RAM+AYREG+AmplA),A
        LD (RAM+AYREG+AmplB),HL

; update AY-3-8910 registers
ROUT    LD   HL,RAM+AYREG
        LD   BC,$00F7   ; C = addr port $F7 (B = I/O addr bits 15:8)
        XOR  A          ; start at register 0
.loop   OUT  (C),A      ; send AY register address
        DEC  C          ; data port $F6
        OUTI            ; write data to register
        INC  C          ; addr port $F7
        INC  A
        CP   13         ; loop until done registers 0 to 12
        JR   NZ,.loop
        OUT  (C),A      ; send register 13 address
        LD   A,(HL)     ; get register 13 data
        AND  A
        RET  M          ; if negative then done
        DEC  C          ; data port $F6
        OUT (C),A       ; write register 13 data
        RET


NTBL    DW $0EF8*2,$0E10*2,$0D60*2,$0C80*2,$0BD8*2,$0B28*2
        DW $0A88*2,$09F0*2,$0960*2,$08E0*2,$0858*2,$07E0*2

;*************************************************************
;                          Data
;*************************************************************

;PT2_DATA ds VAR.size  ; add this line to merge data and code
RAM = PT2_DATA

; start a structure definition
STRUCTURE MACRO name,offset
`name`_offset EQU offset
_COUNT    SET   offset
          ENDM

; allocate 1 byte in structure
BYTE      MACRO name
name      EQU   _COUNT
_COUNT    SET   _COUNT+1
          ENDM
; allocate 2 bytes in structure
WORD      MACRO name
name      EQU _COUNT
_COUNT    SET _COUNT+2
          ENDM

; allocate multiple bytes
STRUCT    MACRO name,size
name      EQU   _COUNT
_COUNT    SET   _COUNT+size
          ENDM

; finish defining structure
ENDSTRUCT MACRO name
`name`.size = _COUNT-`name`_offset
          ENDM

; AY-3-8910 registers
 Structure AYR,0
      WORD ToneA
      WORD ToneB
      WORD ToneC
      BYTE Noise
      BYTE Mixer
      BYTE AmplA
      BYTE AmplB
      BYTE AmplC
      WORD Env
      BYTE EnvTp
 Endstruct AYR

 Structure CHP,0
     byte CHP.PsInSm
     byte CHP.PsInOr
     word CHP.CrTnSl
     byte CHP.Flags
     byte CHP.TSlStp
     word CHP.SamPtr
     word CHP.OrnPtr
     byte CHP.SlToNt
     byte CHP.Note
     word CHP.TnDelt
     byte CHP.Skip
     byte CHP.SkpCnt
     byte CHP.Volume
     byte CHP.Env_En
     byte CHP.AddToN
 endstruct CHP

; one big structure for all our variables
 Structure VAR,0
      byte SETUP           ; Set bit0 = don't loop. bit7 set = end of module.
      word CrPsPtr         ; Pointer to current position in PT2 module.
; static variables: set up in INIT.
      word MODADDR
      word MDADDR2
      word CSP             ; stack pointer save
      word AdInPtA
      word LPosPtr
      word PatsPtr
      word AdInPtB
      word AdInPtC
      byte Delay
; dynamic variables: cleared after initialization
      byte DelyCnt
    struct ChanA,CHP.Size
    struct ChanB,CHP.Size
    struct ChanC,CHP.Size
    struct AYREG,AYR.size  ; data sent to AY registers
; tables: generated in INIT
    struct VT_,256         ; volume tables
    struct NT_,192         ; note tables
 Endstruct VAR

VAR0STRT EQU RAM+DelyCnt   ; start of variables cleared after init.
VAR0END  EQU RAM+VT_+16    ; end of variables cleared after init.
EMPTYORN EQU RAM+VT_+31    ; 1,0,0 sequence

